@@ -65,7 +65,7 @@ typedef struct _GstXingSeekEntry
 } GstXingSeekEntry;
 
 static inline GstXingSeekEntry *
-gst_xing_seek_entry_new (void)
+gst_xing_seek_entry_new ()
 {
   return g_slice_new (GstXingSeekEntry);
 }
@@ -165,7 +165,7 @@ parse_header (guint32 header, guint * ret_size, guint * ret_spf,
   bitrate = (header >> 12) & 0xF;
   bitrate = mp3types_bitrates[lsf][layer - 1][bitrate] * 1000;
   if (bitrate == 0)
-    return FALSE;
+    return 0;
 
   samplerate = (header >> 10) & 0x3;
   samplerate = mp3types_freqs[lsf + mpg25][samplerate];
@@ -390,17 +390,20 @@ gst_xing_mux_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_xing_mux_src_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_xing_mux_sink_template);
-
-  GST_DEBUG_CATEGORY_INIT (xing_mux_debug, "xingmux", 0, "Xing Header Muxer");
-
-  gst_element_class_set_details_simple (element_class, "MP3 Xing muxer",
+  static const GstElementDetails gst_xing_mux_details =
+      GST_ELEMENT_DETAILS ("MP3 Xing muxer",
       "Formatter/Metadata",
       "Adds a Xing header to the beginning of a VBR MP3 file",
       "Christophe Fergeau <teuf@gnome.org>");
+
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_xing_mux_src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_xing_mux_sink_template));
+
+  GST_DEBUG_CATEGORY_INIT (xing_mux_debug, "xingmux", 0, "Xing Header Muxer");
+
+  gst_element_class_set_details (element_class, &gst_xing_mux_details);
 }
 
 static void
@@ -534,7 +537,7 @@ gst_xing_mux_chain (GstPad * pad, GstBuffer * buffer)
 
         xing_header_size = GST_BUFFER_SIZE (xing_header);
 
-        if ((ret = gst_pad_push (xing->srcpad, xing_header)) != GST_FLOW_OK) {
+        if (GST_FLOW_IS_FATAL (ret = gst_pad_push (xing->srcpad, xing_header))) {
           GST_ERROR_OBJECT (xing, "Failed to push Xing header: %s",
               gst_flow_get_name (ret));
           gst_buffer_unref (xing_header);
@@ -570,7 +573,7 @@ gst_xing_mux_chain (GstPad * pad, GstBuffer * buffer)
     else
       xing->duration += duration;
 
-    if ((ret = gst_pad_push (xing->srcpad, outbuf)) != GST_FLOW_OK) {
+    if (GST_FLOW_IS_FATAL (ret = gst_pad_push (xing->srcpad, outbuf))) {
       GST_ERROR_OBJECT (xing, "Failed to push MP3 frame: %s",
           gst_flow_get_name (ret));
       return ret;
@@ -637,7 +640,7 @@ gst_xing_mux_sink_event (GstPad * pad, GstEvent * event)
 
             GST_INFO ("Writing real Xing header to beginning of stream");
 
-            if ((ret = gst_pad_push (xing->srcpad, header)) != GST_FLOW_OK)
+            if (GST_FLOW_IS_FATAL (ret = gst_pad_push (xing->srcpad, header)))
               GST_WARNING ("Failed to push updated Xing header: %s\n",
                   gst_flow_get_name (ret));
           }
